@@ -6,9 +6,9 @@ from collections import namedtuple
 from nose.tools import nottest
 from unittest import TestCase
 
-from . import six
+from . import compat
 
-if six.PY3:
+if compat.PY3:
     def new_instancemethod(f, *args):
         return f
 
@@ -73,7 +73,7 @@ class param(_param):
             """
         if isinstance(args, param):
             return args
-        if isinstance(args, six.string_types):
+        if isinstance(args, compat.string_types):
             args = (args, )
         return cls(*args)
 
@@ -195,7 +195,7 @@ class parameterized(object):
         return input_values
 
     @classmethod
-    def expand(cls, input):
+    def expand(cls, input, testcase_func_name=None):
         """ A "brute force" method of parameterizing test cases. Creates new
             test cases and injects them into the namespace that the wrapped
             function is being defined in. Useful for parameterizing tests in
@@ -220,17 +220,25 @@ class parameterized(object):
             get_input = cls.input_as_callable(input)
             for num, args in enumerate(get_input()):
                 p = param.from_decorator(args)
-                name_suffix = "_%s" %(num, )
-                if len(p.args) > 0 and isinstance(p.args[0], six.string_types):
-                    name_suffix += "_" + cls.to_safe_name(p.args[0])
-                name = base_name + name_suffix
+                if testcase_func_name:
+                    # Caller wants to over-ride default test case func/method naming scheme.
+                    name = testcase_func_name(f, num, p)
+                else:
+                    name_suffix = "_%s" %(num, )
+                    if len(p.args) > 0 and isinstance(p.args[0], compat.string_types):
+                        name_suffix += "_" + cls.to_safe_name(p.args[0])
+                    name = base_name + name_suffix
                 frame_locals[name] = cls.param_as_standalone_func(p, f, name)
             return nottest(f)
         return parameterized_expand_wrapper
 
     @classmethod
     def param_as_standalone_func(cls, p, func, name):
-        standalone_func = lambda *a: func(*(a + p.args), **p.kwargs)
+
+        @wraps(func)
+        def standalone_func(*a):
+            return func(*(a + p.args), **p.kwargs)
+
         standalone_func.__name__ = name
         return standalone_func
 
